@@ -16,7 +16,7 @@
 #'
 #' @importFrom ggplot2 ggplot geom_histogram aes theme_bw stat_qq labs geom_qq_line geom_point
 #' @importFrom stats fitted qnorm quantile residuals sd shapiro.test
-#' @importFrom cowplot plot_grid add_sub
+#' @importFrom patchwork wrap_plots plot_layout plot_annotation plot_spacer
 #'
 #' @examples
 #' dat.aov <- aov(Petal.Length ~ Petal.Width, data = iris)
@@ -41,10 +41,16 @@ resplot <- function(model.obj, shapiro = TRUE, call = FALSE,
 
     for (i in 1:model_info$facet) {
         group_residuals <- residuals_df[residuals_df$lvl == i, ]
-        group_residuals$stdres <- group_residuals$residuals /
-            (sd(group_residuals$residuals, na.rm = TRUE) *
-                 sqrt((sum(!is.na(group_residuals$residuals)) - 1) /
-                          sum(!is.na(group_residuals$residuals))))
+
+        n_non_missing <- sum(!is.na(group_residuals$residuals))
+        sd_resids <- sd(group_residuals$residuals, na.rm = TRUE)
+        denom <- sd_resids * sqrt((n_non_missing - 1) / n_non_missing)
+
+        group_residuals$stdres <- if (is.finite(denom) && denom > 0) {
+            group_residuals$residuals / denom
+        } else {
+            rep(NA_real_, nrow(group_residuals))
+        }
 
         # Create individual plots
         plots <- create_diagnostic_plots(group_residuals, axes.size, label.size)
@@ -68,10 +74,11 @@ resplot <- function(model.obj, shapiro = TRUE, call = FALSE,
 
     # Handle output formatting based on facet structure
     result <- format_output_resplot(output,
-        model_info$facet,
-        model_info$facet_name,
-        onepage,
-        onepage_cols
+                                    model_info$facet,
+                                    model_info$facet_name,
+                                    onepage,
+                                    onepage_cols,
+                                    label.size
     )
 
     return(result)
@@ -84,13 +91,23 @@ resplot <- function(model.obj, shapiro = TRUE, call = FALSE,
 #' @keywords internal
 create_diagnostic_plots <- function(group_residuals, axes.size, label.size) {
 
+    # ggplot2 stats/geoms warn when non-finite values are dropped during build/print.
+    # Filter here so downstream printing (e.g., vdiffr) stays clean.
+    group_residuals_clean <- group_residuals[
+        is.finite(group_residuals$stdres) & !is.na(group_residuals$stdres),
+    ]
+
+    group_scatter_clean <- group_residuals_clean[
+        is.finite(group_residuals_clean$fitted) & !is.na(group_residuals_clean$fitted),
+    ]
+
     # Set histogram x axis width
-    max_val <- max(abs(group_residuals$stdres), na.rm = TRUE)
+    max_val <- max(abs(group_residuals_clean$stdres), na.rm = TRUE)
     max_val <- ifelse(max_val > 3.5, max_val+0.25, 3.5)
     breaks_seq <- seq(0.25, max_val, by = 0.5)
     breaks_seq <- c(-rev(breaks_seq), breaks_seq)
 
-    a <- ggplot2::ggplot(data = group_residuals, mapping = ggplot2::aes(x = stdres)) +
+    a <- ggplot2::ggplot(data = group_residuals_clean, mapping = ggplot2::aes(x = stdres)) +
         ggplot2::geom_histogram(
             bins = ifelse(nrow(group_residuals) < 31, 7, 11),
             fill = "aquamarine3",
@@ -98,9 +115,10 @@ create_diagnostic_plots <- function(group_residuals, axes.size, label.size) {
             breaks = breaks_seq
         ) +
         ggplot2::theme_bw(base_size = axes.size) +
-        ggplot2::labs(y = "Frequency", x = "Standardised Residual")
+        ggplot2::labs(y = "Frequency", x = "Standardised Residual", tag = "A") +
+        ggplot2::theme(plot.tag = ggplot2::element_text(size = label.size))
 
-    b <- ggplot2::ggplot(group_residuals, ggplot2::aes(sample = stdres)) +
+    b <- ggplot2::ggplot(group_residuals_clean, ggplot2::aes(sample = stdres)) +
         ggplot2::geom_qq(
             colour = "black",
             fill = "aquamarine3",
@@ -109,10 +127,11 @@ create_diagnostic_plots <- function(group_residuals, axes.size, label.size) {
         ) +
         ggplot2::geom_qq_line() +
         ggplot2::theme_bw(base_size = axes.size) +
-        ggplot2::labs(y = "Standardised Residual", x = "Theoretical")
+        ggplot2::labs(y = "Standardised Residual", x = "Theoretical", tag = "B") +
+        ggplot2::theme(plot.tag = ggplot2::element_text(size = label.size))
 
     c <- ggplot2::ggplot(
-        data = group_residuals,
+        data = group_scatter_clean,
         mapping = ggplot2::aes(x = fitted, y = stdres)
     ) +
         ggplot2::geom_point(
@@ -122,7 +141,8 @@ create_diagnostic_plots <- function(group_residuals, axes.size, label.size) {
             shape = 21
         ) +
         ggplot2::theme_bw(base_size = axes.size) +
-        ggplot2::labs(y = "Standardised Residual", x = "Fitted Value")
+        ggplot2::labs(y = "Standardised Residual", x = "Fitted Value", tag = "C") +
+        ggplot2::theme(plot.tag = ggplot2::element_text(size = label.size))
 
     list(histogram = a, qq = b, scatter = c)
 }
@@ -143,14 +163,14 @@ shapiro_test <- function(group_residuals) {
 
     shap <- shapiro.test(group_residuals$residuals)
     n <- length(group_residuals$residuals)
-    list(
-        text1 = paste(shap$method, "p-value:", round(shap$p.value, 4)),
-        text2 = ifelse(
-            shap$p.value > 0.05,
-            paste0("The residuals appear to be normally distributed. (n = ", n, ")"),
-            paste0("The residuals do not appear to be normally distributed. (n = ", n, ")")
-        )
+    output <- list(text1 = paste(shap$method, "p-value:", round(shap$p.value, 4)),
+                   text2 = ifelse(
+                       shap$p.value > 0.05,
+                       paste0("The residuals appear to be normally distributed. (n = ", n, ")"),
+                       paste0("The residuals do not appear to be normally distributed. (n = ", n, ")"))
     )
+
+    return(output)
 }
 
 #' Combine plots into final output
@@ -162,67 +182,59 @@ shapiro_test <- function(group_residuals) {
 #' @param label.size Size of labels
 #' @keywords internal
 combine_plots <- function(plots, shapiro_result, model_call, call, call.size, label.size) {
-    top_row <- cowplot::plot_grid(
-        plots$histogram,
-        plots$qq,
-        ncol = 2,
-        labels = c("A", "B"),
-        label_size = label.size
-    )
-
+    # Build the complete layout using patchwork's design specification
     if (!is.null(shapiro_result)) {
-        bottom_row <- cowplot::plot_grid(
-            NULL,
-            cowplot::add_sub(
-                cowplot::add_sub(
-                    plots$scatter,
-                    shapiro_result$text1,
-                    size = 11,
-                    vjust = 1.2
-                ),
-                shapiro_result$text2,
-                size = 9,
-                vjust = 0.2
-            ),
-            NULL,
-            ncol = 3,
-            rel_widths = c(0.25, 0.5, 0.25),
-            labels = c("", "C", ""),
-            label_size = label.size,
-            hjust = 1
-        )
+        shapiro_plot <- ggplot2::ggplot() +
+            ggplot2::annotate(
+                "text",
+                x = 0, y = 0.75,
+                label = shapiro_result$text1,
+                size = 3.5,
+                hjust = 0.5,
+                vjust = 1
+            ) +
+            ggplot2::annotate(
+                "text",
+                x = 0, y = 0.25,
+                label = shapiro_result$text2,
+                size = 3.0,
+                hjust = 0.5,
+                vjust = 0
+            ) +
+            ggplot2::theme_void() +
+            ggplot2::coord_cartesian(xlim = c(-1, 1), ylim = c(0, 1), clip = "off")
+
+        # Create complete layout with all elements including Shapiro
+        combined <- plots$histogram + plots$qq +
+            plots$scatter +
+            shapiro_plot +
+            patchwork::plot_layout(
+                design = "AAAAABBBBB
+                          ##CCCCCC##
+                          DDDDDDDDDD",
+                heights = c(1, 1, 0.4)
+            )
     } else {
-        bottom_row <- cowplot::plot_grid(
-            NULL,
-            plots$scatter,
-            NULL,
-            ncol = 3,
-            rel_widths = c(0.25, 0.5, 0.25),
-            labels = c("", "C", ""),
-            hjust = -1,
-            label_size = label.size
-        )
+        # Create layout without Shapiro text
+        combined <- plots$histogram + plots$qq + plots$scatter +
+            patchwork::plot_layout(
+                design = "AABB
+                          #CC#"
+            )
     }
 
+    # Add model call if requested
     if (call && !is.null(model_call)) {
-        title <- cowplot::ggdraw() +
-            cowplot::draw_label(model_call, size = call.size, hjust = 0.5)
-        call_row <- cowplot::plot_grid(title, ncol = 1)
-        cowplot::plot_grid(
-            call_row,
-            top_row,
-            bottom_row,
-            ncol = 1,
-            rel_heights = c(0.1, 0.4, 0.4)
-        )
-    } else {
-        cowplot::plot_grid(
-            top_row,
-            bottom_row,
-            ncol = 1,
-            rel_heights = c(0.4, 0.4)
-        )
+        combined <- combined +
+            patchwork::plot_annotation(
+                title = model_call,
+                theme = ggplot2::theme(
+                    plot.title = ggplot2::element_text(size = call.size, hjust = 0.5)
+                )
+            )
     }
+
+    combined
 }
 
 #' Format final output based on facet structure
@@ -232,36 +244,40 @@ combine_plots <- function(plots, shapiro_result, model_call, call, call.size, la
 #' @param onepage Logical, combine plots on one page
 #' @param onepage_cols Number of columns for onepage layout
 #' @keywords internal
-format_output_resplot <- function(output, facet, facet_name, onepage, onepage_cols) {
+format_output_resplot <- function(output, facet, facet_name, onepage, onepage_cols, label.size) {
     if (facet > 1) {
         names(output) <- facet_name
 
         if (onepage) {
-            # Validate onepage_cols
             onepage_cols <- min(max(1, onepage_cols), facet)
-
-            # Calculate number of pages needed based on onepage_cols
             plots_per_page <- onepage_cols * ceiling(6 / onepage_cols)
             n_pages <- ceiling(facet / plots_per_page)
             pages <- vector("list", n_pages)
 
             for (page in 1:n_pages) {
-                # Get index range for current page
                 start_idx <- (page - 1) * plots_per_page + 1
                 end_idx <- min(page * plots_per_page, facet)
-
-                # Calculate grid dimensions for current page
                 n_plots_on_page <- end_idx - start_idx + 1
                 n_cols <- min(onepage_cols, n_plots_on_page)
                 n_rows <- ceiling(n_plots_on_page / n_cols)
 
-                # Create combined plot for current page
-                pages[[page]] <- cowplot::plot_grid(
-                    plotlist = output[start_idx:end_idx],
+                # Add titles to each plot group using wrap_elements
+                plots_with_titles <- lapply(start_idx:end_idx, function(i) {
+                    patchwork::wrap_elements(full = output[[i]]) +
+                        ggplot2::ggtitle(facet_name[i]) +
+                        ggplot2::theme(
+                            plot.title = ggplot2::element_text(
+                                face = "bold",
+                                size = 14,
+                                hjust = 0.5
+                            )
+                        )
+                })
+
+                pages[[page]] <- patchwork::wrap_plots(
+                    plots_with_titles,
                     ncol = n_cols,
-                    nrow = n_rows,
-                    scale = 0.90,
-                    labels = facet_name[start_idx:end_idx]
+                    nrow = n_rows
                 )
             }
             return(pages)
@@ -300,15 +316,13 @@ NULL
 #' For `resplt`, use [resplot()].
 #'
 #' @export
-resplt <- function(
-        model.obj,
-        shapiro = TRUE,
-        call = FALSE,
-        label.size = 10,
-        axes.size = 10,
-        call.size = 9,
-        mod.obj
-) {
+resplt <- function(model.obj,
+                   shapiro = TRUE,
+                   call = FALSE,
+                   label.size = 10,
+                   axes.size = 10,
+                   call.size = 9,
+                   mod.obj) {
     .Deprecated(
         msg = "resplt has been deprecated in version 1.0.1 and will be removed in a future version.\nPlease use resplot() instead."
     )
